from __future__ import print_function

import argparse
import sys

from . import encode_orgkey, decode_orgkey
from .exceptions import InvalidOrgKeyError


def main():
    args = parse_args()
    args.func(args)


def parse_args():
    parser = argparse.ArgumentParser(
        formatter_class=argparse.ArgumentDefaultsHelpFormatter
    )

    subparsers = parser.add_subparsers(help='commands', dest='cmd')
    subparsers.required = True

    encode_parser = subparsers.add_parser('encode', help='Encode org key')
    encode_parser.add_argument(
        '--env',
        action='store',
        help='Environment name (e.g. dev01, prod05)',
        required=True
    )
    encode_parser.add_argument(
        '--id',
        action='store',
        dest='org_id',
        type=int,
        help='Environment-specific organization ID',
        required=True
    )
    encode_parser.set_defaults(func=encode)

    decode_parser = subparsers.add_parser('decode', help='Decode org key')
    decode_parser.add_argument(
        'orgkey',
        action='store',
        nargs='*',
        help='Globally unique organization key'
    )
    decode_parser.set_defaults(func=decode)

    return parser.parse_args()


def decode(args):
    def make_result(env, org_id):
        return '{env} - {org_id}'.format(
            env=env,
            org_id=org_id
        )

    if len(args.orgkey) == 1:
        try:
            decoded = decode_orgkey(args.orgkey[0])
        except InvalidOrgKeyError:
            sys.exit(1)
        else:
            print(make_result(*decoded))
    else:
        errors = 0
        for orgkey in args.orgkey:
            try:
                decoded = decode_orgkey(orgkey)
                result = make_result(*decoded)
            except InvalidOrgKeyError:
                result = u'<invalid>'
                errors += 1
            print(u'{orgkey}: {result}'.format(
                orgkey=orgkey,
                result=result
            ))
        if errors != 0:
            sys.exit(1)


def encode(args):
    print(encode_orgkey(args.env, args.org_id))
